###############################################################################
################## SCRIPT FOR LOADING ALL SCORECARD FUNCTIONS #################
###############################################################################

##### Script requires module Phantomjs, currently loaded in WS
##### If using Nord3, the module needs to be loaded before running the code
##### To load Phantomjs in Nord3: module load Phantomjs

##### To open R in Nord3: 
#  module load R/4.1.2-foss-2019b
#  module load OpenMPI/4.0.5-GCC-8.3.0-nord3-v2
#  R

## Cleaning
rm(list = ls()); gc()

### Set the working directory containing the scripts
setwd("/esarchive/scratch/nmilders/gitlab/git_clones/s2s-suite/")


#' Execute scorecards 
#' 
#'@description Scorecards function load scorecard metrics, aggregate metrics and
#'  visualize single and multi-system/reference scorecards
#'
#'@param system a vector of character strings defining the names of the 
#'  system names following the archive.yml format from verification suite 
#'  (eg. c('ecmwfs5', 'cmccs35', 'dwds21'))
#'@param reference a vector of character strings defining the names of 
#'  the references following the archive.yml format from verification suite 
#'  Pending to be test with more than one.
#'@param var a character string following the format of variable-dictionary.yml 
#'  from verification suite (e.g. - 'tas', 'prlr', 'psl', 'sfcWind') 
#'  (TO DO: multiple variables)
#'@param start.year a numeric indicating the start year of the reference period
#'@param end.year a numeric indicating the end year of the reference period
#'@param start.months a vector of character strings indicating the start months.
#'@param forecast.months a vector of character strings indicating the forecast.
#' months
#'@param metric.aggregation a character indicating whether the skill score RPS 
#' and CRPSS are calculated from aggregated scores or aggregated skill score 
#' directly, either 'score' or 'skill' respectively.
#'@param metrics.load a vector of character strings defining the metrics to load 
#'  from the data files, following the metric names in the verification suite. 
#'  If metric.aggregation = 'score' then the score and score_clim should both be
#'  loaded to calculate the corresponding skill score, e.g. rps and rps_clim to 
#'  calculate rpss from the aggregated score.
#'@param metrics.visualize a vector of character stings defining the metrics to 
#'  visualize in the scorecard. This can be different to metrics.load in the 
#'  case that metric.aggregation = 'score'. In this case when skills are 
#'  calculated from the aggregated score the resulting metric names will be 
#'  rpss_score_aggr or crpss_score_aggr.
#'@param cross.validation a logical indicating whether cross-validation was used
#'  to calculate anomalies.
#'@param regions is a named list of vectors containing the desired regions to 
#'  analyze. For each region the following should be specified in this order:
#'  lon_min, lon_max, lat_min, lat_max.
#'@param input.path a character string indicating the path where metrics output 
#'  files from verification suite where store (or any other compatible files)
#'@param fileout a character string indicating the path of the location to save 
#'  the scorecard visualizations.
#'@param ncores a numeric indicating the number of cores to use for the 
#'  aggregation of metrics calculation.
#' 
#'@return
#'  returns scorecard visualizations in the -png format, saved in the location 
#'  defined in fileout.
#' 
#'@example
#'execute_scorecards <- SC_execute_scorecards(system = c('ecmwfs5','dwds21'),
#'                                            reference = 'era5',
#'                                            var = 'tas',
#'                                            start.year = 1993,
#'                                            end.year = 2016,
#'                                            start.months = 1:12,
#'                                            forecast.months = 1:6,
#'                                            metric.aggregation = 'score',
#'                                            metrics.load =  c('mean_bias', 'enscorr', 'rps', 'rps_clim', 'crpss', 'enssprerr'),
#'                                            metrics.visualize =  c('mean_bias', 'enscorr', 'rpss_score_aggr', 'crpss', 'enssprerr'),
#'                                            cross.validation = TRUE,
#'                                            regions = list('Tropics' = c(lon.min = 0, lon.max = 360, lat.min = -30, lat.max = 30),
#'                                                            'Extra-tropical NH' = c(lon.min = -0, lon.max = 360, lat.min = 30, lat.max = 90),
#'                                                            'Extra-tropical SH' = c(lon.min = 0, lon.max = 360, lat.min = -30, lat.max = -90))
#'                                            input.path = '/esarchive/scratch/nmilders/scorecards_data/input_data'
#'                                            fileout = '/esarchive/scratch/nmilders/saved_scorecards/cross-validation',
#'                                            ncores = 4)




Scorecards <- SC_execute_scorecards(system = c('ecmwfs5','cmccs35', 'dwds21'),
                                    reference = 'era5',
                                    var = 'tas', # tas, prlr, psl, sfcWind
                                    start.year = 1993,
                                    end.year = 2016,
                                    start.months = 1:12,
                                    forecast.months = 1:6,
                                    # metric.aggregation = 'score',
                                    # metrics.load =  c('mean_bias', 'enscorr', 'rps', 'rps_clim', 'crps', 'crps_clim', 'enssprerr'),
                                    # metrics.visualize =  c('mean_bias', 'enscorr', 'rpss_score_aggr', 'crpss_score_aggr', 'enssprerr'),
                                    metric.aggregation = 'skill',
                                    metrics.load =  c('mean_bias', 'enscorr', 'rpss', 'crpss', 'enssprerr'),
                                    metrics.visualize =  c('mean_bias', 'enscorr', 'rpss', 'crpss', 'enssprerr'),
                                    regions = list('Tropics' = c(lon.min = 0, lon.max = 360, lat.min = -30, lat.max = 30),
                                                   'Extra-tropical NH' = c(lon.min = -0, lon.max = 360, lat.min = 30, lat.max = 90),
                                                   'Extra-tropical SH' = c(lon.min = 0, lon.max = 360, lat.min = -30, lat.max = -90)),
                                    # cross.validation = TRUE,
                                    # input.path = '/esarchive/scratch/nmilders/scorecards_data/input_data/cross_validation',
                                    # fileout = '/esarchive/scratch/nmilders/saved_scorecards/cross_validation',
                                    cross.validation = FALSE,
                                    input.path = '/esarchive/scratch/nmilders/scorecards_data/input_data/not_cross_validation',
                                    fileout = '/esarchive/scratch/nmilders/saved_scorecards/not_cross_validation',
                                    ncores = 4)

## For testing
# system <- c('ecmwfs5')
# reference <- 'era5'
# var <- 'tas' # tas, prlr, psl, sfcWind
# start.year <- 1993
# end.year <- 2016
# start.months <- 1:12
# forecast.months <- 1:6
# # metric.aggregation <- 'score',
# # metrics.load <-  c('mean_bias', 'enscorr', 'rps', 'rps_clim', 'crps', 'crps_clim', 'enssprerr'),
# # metrics.visualize <-  c('mean_bias', 'enscorr', 'rpss_score_aggr', 'crpss_score_aggr', 'enssprerr'),
# metric.aggregation <- 'skill'
# metrics.load <-  c('mean_bias', 'enscorr', 'rpss', 'crpss', 'enssprerr')
# metrics.visualize <-  c('mean_bias', 'enscorr', 'rpss', 'crpss', 'enssprerr')
# regions <- list('Tropics' = c(lon.min <- 0, lon.max <- 360, lat.min <- -30, lat.max <- 30),
#                 'Extra-tropical NH' = c(lon.min <- -0, lon.max <- 360, lat.min <- 30, lat.max <- 90),
#                 'Extra-tropical SH' = c(lon.min <- 0, lon.max <- 360, lat.min <- -30, lat.max <- -90))
# # regions <- list('Tropics (30ºN-30ºN)' = c(lon.min <- 0, lon.max <- 360, lat.min <- -30, lat.max <- 30),
# #                 'Extra-tropical NH (30ºN-90ºN)' = c(lon.min <- -0, lon.max <- 360, lat.min <- 30, lat.max <- 90),
# #                 'Extra-tropical SH (90ºS-30ºS)' = c(lon.min <- 0, lon.max <- 360, lat.min <- -30, lat.max <- -90))
# 
# # cross.validation <- TRUE,
# # input.path <- '/esarchive/scratch/nmilders/scorecards_data/input_data/cross_validation',
# # fileout <- '/esarchive/scratch/nmilders/saved_scorecards/cross_validation',
# cross.validation <- FALSE
# input.path <- '/esarchive/scratch/nmilders/scorecards_data/input_data/not_cross_validation'
# fileout <- '/esarchive/scratch/nmilders/saved_scorecards/not_cross_validation'
# ncores <- 4




SC_execute_scorecards <- function(system,
                                  reference,
                                  var,
                                  start.year,
                                  end.year,
                                  start.months,
                                  forecast.months,
                                  metric.aggregation,
                                  metrics.load,
                                  metrics.visualize,
                                  cross.validation,
                                  regions,
                                  input.path,
                                  fileout,
                                  ncores){
  
  
  
  ##### Load libraries ######
  library(ncdf4) ## to load netCDF files
  library(formattable) ## to plot horizontal color bars
  library(kableExtra) ## to print data in formatted tables
  library(stringr) ## for manipulating capitals in the figure title
  library(s2dv) ## for function ColorBar() 
  library(abind) ## for function abind()
  library(data.table) ## for function %like%
  library(ClimProjDiags) ## for WeightedMean function
  library(yaml) ## to read yaml config files
  
  ## Load data files
  source("/esarchive/scratch/nmilders/gitlab/git_clones/s2s-suite/modules/Scorecards/SC_1_load_metrics.R")
  loaded_metrics <- SC_load_metrics(system = system,
                                    reference = reference,
                                    var = var,
                                    start.year = start.year,
                                    end.year = end.year,
                                    metrics =  metrics.load,
                                    start.months = sprintf("%02d", start.months),
                                    forecast.months = forecast.months,
                                    input.path = input.path)
  
  
  ## Calculate weighted mean of spatial aggregation
  source("/esarchive/scratch/nmilders/gitlab/git_clones/s2s-suite/modules/Scorecards/SC_2_spatial_aggregation.R")
  aggregated_metrics <- SC_spatial_aggregation(loaded_metrics,
                                               regions = regions,
                                               metric.aggregation = metric.aggregation,
                                               ncores = ncores)
  
  ## Create simple scorecard tables 
  ## (one system only)
  ## Metrics input must be in the same order as function SC_spatial_aggregation
  # source("/esarchive/scratch/nmilders/gitlab/git_clones/s2s-suite/modules/Scorecards/SC_3_create_scorecard_simple.R")
  # scorecard_simple <- SC_create_table_simple(data = aggregated_metrics,
  #                                            system = system,
  #                                            reference = reference,
  #                                            var = var,
  #                                            start.year = start.year,
  #                                            end.year = end.year,
  #                                            start.months = start.months,
  #                                            forecast.months = forecast.months,
  #                                            region.names = attributes(regions)$names,
  #                                            metric.aggregation = metric.aggregation,
  #                                            metrics = metrics.visualize,
  #                                            cross.validation = cross.validation,
  #                                            fileout = fileout)
  
  
  ## Create multi system/reference scorecard tables
  ## (multiple systems with one reference or one system with multiple references)
  ## Metrics input must be in the same order as function SC_spatial_aggregation
  if(length(system) >  1 || length(reference) >  1){
    source("/esarchive/scratch/nmilders/gitlab/git_clones/s2s-suite/modules/Scorecards/SC_4_create_scorecard_multi.R")
    scorecard_multi <- SC_create_table_multi(data = aggregated_metrics,
                                             system = system,
                                             reference = reference,
                                             var = var,
                                             start.year = start.year,
                                             end.year = end.year,
                                             start.months = start.months,
                                             forecast.months = forecast.months,
                                             region.names = attributes(regions)$names,
                                             metric.aggregation = metric.aggregation,
                                             metrics = metrics.visualize,
                                             cross.validation = cross.validation,
                                             fileout = fileout)
  } ## close if
  
} ## close function    

